@@ -61,8 +61,7 @@ void gMySQLBackend::reconnect()
                                          getArg("group"),
                                          mustDo("innodb-read-committed"),
                                          getArgAsNum("timeout"),
-                                         mustDo("thread-cleanup"),
-                                         mustDo("ssl"))));
+                                         mustDo("thread-cleanup"))));
   allocateStatements();
 }
 
@@ -95,6 +94,9 @@ public:
     declare(suffix, "any-query", "Any query", record_query + " disabled=0 and name=?");
     declare(suffix, "any-id-query", "Any with ID query", record_query + " disabled=0 and name=? and domain_id=?");
 
+    declare(suffix, "api-id-query", "API basic with ID query", record_query + " (disabled=0 or ?) and type=? and name=? and domain_id=?");
+    declare(suffix, "api-any-id-query", "API any with ID query", record_query + " (disabled=0 or ?) and name=? and domain_id=?");
+
     declare(suffix, "list-query", "AXFR query", "SELECT content,ttl,prio,type,domain_id,disabled,name,auth,ordername FROM records WHERE (disabled=0 OR ?) and domain_id=? order by name, type");
     declare(suffix, "list-subzone-query", "Subzone listing", record_query + " disabled=0 and (name=? OR name like ?) and domain_id=?");
 
