@@ -25,6 +25,14 @@
 #endif
 #include "remotebackend.hh"
 
+// The following requirement guarantees UnknownDomainID will get output as "-1"
+// in the Json data for compatibility.
+static_assert(std::is_signed<domainid_t>::value);
+
+// The following requirement is a consequency of JsonInt not able to store
+// anything larger than "int".
+static_assert(sizeof(domainid_t) <= sizeof(int));
+
 static const char* kBackendId = "[RemoteBackend]";
 
 /**
@@ -193,7 +201,7 @@ int RemoteBackend::build()
  * The functions here are just remote json stubs that send and receive the method call
  * data is mainly left alone, some defaults are assumed.
  */
-void RemoteBackend::lookup(const QType& qtype, const DNSName& qdomain, int zoneId, DNSPacket* pkt_p)
+void RemoteBackend::lookup(const QType& qtype, const DNSName& qdomain, domainid_t zoneId, DNSPacket* pkt_p)
 {
   if (d_index != -1) {
     throw PDNSException("Attempt to lookup while one running");
@@ -225,7 +233,34 @@ void RemoteBackend::lookup(const QType& qtype, const DNSName& qdomain, int zoneI
   d_index = 0;
 }
 
-bool RemoteBackend::list(const DNSName& target, int domain_id, bool include_disabled)
+// Similar to lookup above, but passes an extra include_disabled parameter.
+void RemoteBackend::APILookup(const QType& qtype, const DNSName& qdomain, domainid_t zoneId, bool include_disabled)
+{
+  if (d_index != -1) {
+    throw PDNSException("Attempt to lookup while one running");
+  }
+
+  string localIP = "0.0.0.0";
+  string remoteIP = "0.0.0.0";
+  string realRemote = "0.0.0.0/0";
+
+  Json query = Json::object{
+    {"method", "APILookup"},
+    {"parameters", Json::object{{"qtype", qtype.toString()}, {"qname", qdomain.toString()}, {"remote", remoteIP}, {"local", localIP}, {"real-remote", realRemote}, {"zone-id", zoneId}, {"include-disabled", include_disabled}}}};
+
+  if (!this->send(query) || !this->recv(d_result)) {
+    return;
+  }
+
+  // OK. we have result parameters in result. do not process empty result.
+  if (!d_result["result"].is_array() || d_result["result"].array_items().empty()) {
+    return;
+  }
+
+  d_index = 0;
+}
+
+bool RemoteBackend::list(const ZoneName& target, domainid_t domain_id, bool include_disabled)
 {
   if (d_index != -1) {
     throw PDNSException("Attempt to lookup while one running");
@@ -257,7 +292,7 @@ bool RemoteBackend::get(DNSResourceRecord& rr)
   rr.qclass = QClass::IN;
   rr.content = stringFromJson(d_result["result"][d_index], "content");
   rr.ttl = d_result["result"][d_index]["ttl"].int_value();
-  rr.domain_id = intFromJson(d_result["result"][d_index], "domain_id", -1);
+  rr.domain_id = static_cast<domainid_t>(intFromJson(d_result["result"][d_index], "domain_id", UnknownDomainID));
   if (d_dnssec) {
     rr.auth = (intFromJson(d_result["result"][d_index], "auth", 1) != 0);
   }
@@ -275,7 +310,8 @@ bool RemoteBackend::get(DNSResourceRecord& rr)
   return true;
 }
 
-bool RemoteBackend::getBeforeAndAfterNamesAbsolute(uint32_t id, const DNSName& qname, DNSName& unhashed, DNSName& before, DNSName& after)
+// NOLINTNEXTLINE(readability-identifier-length)
+bool RemoteBackend::getBeforeAndAfterNamesAbsolute(domainid_t id, const DNSName& qname, DNSName& unhashed, DNSName& before, DNSName& after)
 {
   // no point doing dnssec if it's not supported
   if (!d_dnssec) {
@@ -284,7 +320,7 @@ bool RemoteBackend::getBeforeAndAfterNamesAbsolute(uint32_t id, const DNSName& q
 
   Json query = Json::object{
     {"method", "getBeforeAndAfterNamesAbsolute"},
-    {"parameters", Json::object{{"id", Json(static_cast<double>(id))}, {"qname", qname.toString()}}}};
+    {"parameters", Json::object{{"id", Json(id)}, {"qname", qname.toString()}}}};
   Json answer;
 
   if (!this->send(query) || !this->recv(answer)) {
@@ -304,7 +340,7 @@ bool RemoteBackend::getBeforeAndAfterNamesAbsolute(uint32_t id, const DNSName& q
   return true;
 }
 
-bool RemoteBackend::getAllDomainMetadata(const DNSName& name, std::map<std::string, std::vector<std::string>>& meta)
+bool RemoteBackend::getAllDomainMetadata(const ZoneName& name, std::map<std::string, std::vector<std::string>>& meta)
 {
   Json query = Json::object{
     {"method", "getAllDomainMetadata"},
@@ -336,7 +372,7 @@ bool RemoteBackend::getAllDomainMetadata(const DNSName& name, std::map<std::stri
   return true;
 }
 
-bool RemoteBackend::getDomainMetadata(const DNSName& name, const std::string& kind, std::vector<std::string>& meta)
+bool RemoteBackend::getDomainMetadata(const ZoneName& name, const std::string& kind, std::vector<std::string>& meta)
 {
   Json query = Json::object{
     {"method", "getDomainMetadata"},
@@ -366,7 +402,7 @@ bool RemoteBackend::getDomainMetadata(const DNSName& name, const std::string& ki
   return true;
 }
 
-bool RemoteBackend::setDomainMetadata(const DNSName& name, const std::string& kind, const std::vector<std::string>& meta)
+bool RemoteBackend::setDomainMetadata(const ZoneName& name, const std::string& kind, const std::vector<std::string>& meta)
 {
   Json query = Json::object{
     {"method", "setDomainMetadata"},
@@ -380,7 +416,7 @@ bool RemoteBackend::setDomainMetadata(const DNSName& name, const std::string& ki
   return boolFromJson(answer, "result", false);
 }
 
-bool RemoteBackend::getDomainKeys(const DNSName& name, std::vector<DNSBackend::KeyData>& keys)
+bool RemoteBackend::getDomainKeys(const ZoneName& name, std::vector<DNSBackend::KeyData>& keys)
 {
   // no point doing dnssec if it's not supported
   if (!d_dnssec) {
@@ -405,13 +441,13 @@ bool RemoteBackend::getDomainKeys(const DNSName& name, std::vector<DNSBackend::K
     key.active = asBool(jsonKey["active"]);
     key.published = boolFromJson(jsonKey, "published", true);
     key.content = stringFromJson(jsonKey, "content");
-    keys.push_back(key);
+    keys.emplace_back(std::move(key));
   }
 
   return true;
 }
 
-bool RemoteBackend::removeDomainKey(const DNSName& name, unsigned int id)
+bool RemoteBackend::removeDomainKey(const ZoneName& name, unsigned int keyId)
 {
   // no point doing dnssec if it's not supported
   if (!d_dnssec) {
@@ -420,13 +456,13 @@ bool RemoteBackend::removeDomainKey(const DNSName& name, unsigned int id)
 
   Json query = Json::object{
     {"method", "removeDomainKey"},
-    {"parameters", Json::object{{"name", name.toString()}, {"id", static_cast<int>(id)}}}};
+    {"parameters", Json::object{{"name", name.toString()}, {"id", static_cast<int>(keyId)}}}};
 
   Json answer;
   return this->send(query) && this->recv(answer);
 }
 
-bool RemoteBackend::addDomainKey(const DNSName& name, const KeyData& key, int64_t& id)
+bool RemoteBackend::addDomainKey(const ZoneName& name, const KeyData& key, int64_t& keyId)
 {
   // no point doing dnssec if it's not supported
   if (!d_dnssec) {
@@ -442,11 +478,11 @@ bool RemoteBackend::addDomainKey(const DNSName& name, const KeyData& key, int64_
     return false;
   }
 
-  id = answer["result"].int_value();
-  return id >= 0;
+  keyId = answer["result"].int_value();
+  return keyId >= 0;
 }
 
-bool RemoteBackend::activateDomainKey(const DNSName& name, unsigned int id)
+bool RemoteBackend::activateDomainKey(const ZoneName& name, unsigned int keyId)
 {
   // no point doing dnssec if it's not supported
   if (!d_dnssec) {
@@ -455,13 +491,13 @@ bool RemoteBackend::activateDomainKey(const DNSName& name, unsigned int id)
 
   Json query = Json::object{
     {"method", "activateDomainKey"},
-    {"parameters", Json::object{{"name", name.toString()}, {"id", static_cast<int>(id)}}}};
+    {"parameters", Json::object{{"name", name.toString()}, {"id", static_cast<int>(keyId)}}}};
 
   Json answer;
   return this->send(query) && this->recv(answer);
 }
 
-bool RemoteBackend::deactivateDomainKey(const DNSName& name, unsigned int id)
+bool RemoteBackend::deactivateDomainKey(const ZoneName& name, unsigned int keyId)
 {
   // no point doing dnssec if it's not supported
   if (!d_dnssec) {
@@ -470,13 +506,13 @@ bool RemoteBackend::deactivateDomainKey(const DNSName& name, unsigned int id)
 
   Json query = Json::object{
     {"method", "deactivateDomainKey"},
-    {"parameters", Json::object{{"name", name.toString()}, {"id", static_cast<int>(id)}}}};
+    {"parameters", Json::object{{"name", name.toString()}, {"id", static_cast<int>(keyId)}}}};
 
   Json answer;
   return this->send(query) && this->recv(answer);
 }
 
-bool RemoteBackend::publishDomainKey(const DNSName& name, unsigned int id)
+bool RemoteBackend::publishDomainKey(const ZoneName& name, unsigned int keyId)
 {
   // no point doing dnssec if it's not supported
   if (!d_dnssec) {
@@ -485,13 +521,13 @@ bool RemoteBackend::publishDomainKey(const DNSName& name, unsigned int id)
 
   Json query = Json::object{
     {"method", "publishDomainKey"},
-    {"parameters", Json::object{{"name", name.toString()}, {"id", static_cast<int>(id)}}}};
+    {"parameters", Json::object{{"name", name.toString()}, {"id", static_cast<int>(keyId)}}}};
 
   Json answer;
   return this->send(query) && this->recv(answer);
 }
 
-bool RemoteBackend::unpublishDomainKey(const DNSName& name, unsigned int id)
+bool RemoteBackend::unpublishDomainKey(const ZoneName& name, unsigned int keyId)
 {
   // no point doing dnssec if it's not supported
   if (!d_dnssec) {
@@ -500,15 +536,19 @@ bool RemoteBackend::unpublishDomainKey(const DNSName& name, unsigned int id)
 
   Json query = Json::object{
     {"method", "unpublishDomainKey"},
-    {"parameters", Json::object{{"name", name.toString()}, {"id", static_cast<int>(id)}}}};
+    {"parameters", Json::object{{"name", name.toString()}, {"id", static_cast<int>(keyId)}}}};
 
   Json answer;
   return this->send(query) && this->recv(answer);
 }
 
-bool RemoteBackend::doesDNSSEC()
+unsigned int RemoteBackend::getCapabilities()
 {
-  return d_dnssec;
+  unsigned int caps = CAP_DIRECT | CAP_LIST;
+  if (d_dnssec) {
+    caps |= CAP_DNSSEC;
+  }
+  return caps;
 }
 
 bool RemoteBackend::getTSIGKey(const DNSName& name, DNSName& algorithm, std::string& content)
@@ -590,8 +630,8 @@ bool RemoteBackend::getTSIGKeys(std::vector<struct TSIGKey>& keys)
 
 void RemoteBackend::parseDomainInfo(const Json& obj, DomainInfo& di)
 {
-  di.id = intFromJson(obj, "id", -1);
-  di.zone = DNSName(stringFromJson(obj, "zone"));
+  di.id = static_cast<domainid_t>(intFromJson(obj, "id", UnknownDomainID));
+  di.zone = ZoneName(stringFromJson(obj, "zone"));
   for (const auto& primary : obj["masters"].array_items()) {
     di.primaries.emplace_back(primary.string_value(), 53);
   }
@@ -616,7 +656,7 @@ void RemoteBackend::parseDomainInfo(const Json& obj, DomainInfo& di)
   di.backend = this;
 }
 
-bool RemoteBackend::getDomainInfo(const DNSName& domain, DomainInfo& di, bool /* getSerial */)
+bool RemoteBackend::getDomainInfo(const ZoneName& domain, DomainInfo& info, bool /* getSerial */)
 {
   if (domain.empty()) {
     return false;
@@ -631,15 +671,16 @@ bool RemoteBackend::getDomainInfo(const DNSName& domain, DomainInfo& di, bool /*
     return false;
   }
 
-  this->parseDomainInfo(answer["result"], di);
+  this->parseDomainInfo(answer["result"], info);
   return true;
 }
 
-void RemoteBackend::setNotified(uint32_t id, uint32_t serial)
+// NOLINTNEXTLINE(readability-identifier-length)
+void RemoteBackend::setNotified(domainid_t id, uint32_t serial)
 {
   Json query = Json::object{
     {"method", "setNotified"},
-    {"parameters", Json::object{{"id", static_cast<double>(id)}, {"serial", static_cast<double>(serial)}}}};
+    {"parameters", Json::object{{"id", id}, {"serial", static_cast<double>(serial)}}}};
 
   Json answer;
   if (!this->send(query) || !this->recv(answer)) {
@@ -647,7 +688,7 @@ void RemoteBackend::setNotified(uint32_t id, uint32_t serial)
   }
 }
 
-bool RemoteBackend::autoPrimaryBackend(const string& ip, const DNSName& domain, const vector<DNSResourceRecord>& nsset, string* nameserver, string* account, DNSBackend** ddb)
+bool RemoteBackend::autoPrimaryBackend(const string& ipAddress, const ZoneName& domain, const vector<DNSResourceRecord>& nsset, string* nameserver, string* account, DNSBackend** ddb)
 {
   Json::array rrset;
 
@@ -663,7 +704,7 @@ bool RemoteBackend::autoPrimaryBackend(const string& ip, const DNSName& domain,
 
   Json query = Json::object{
     {"method", "superMasterBackend"},
-    {"parameters", Json::object{{"ip", ip}, {"domain", domain.toString()}, {"nsset", rrset}}}};
+    {"parameters", Json::object{{"ip", ipAddress}, {"domain", domain.toString()}, {"nsset", rrset}}}};
 
   *ddb = nullptr;
 
@@ -684,12 +725,12 @@ bool RemoteBackend::autoPrimaryBackend(const string& ip, const DNSName& domain,
   return true;
 }
 
-bool RemoteBackend::createSecondaryDomain(const string& ip, const DNSName& domain, const string& nameserver, const string& account)
+bool RemoteBackend::createSecondaryDomain(const string& ipAddress, const ZoneName& domain, const string& nameserver, const string& account)
 {
   Json query = Json::object{
     {"method", "createSlaveDomain"},
     {"parameters", Json::object{
-                     {"ip", ip},
+                     {"ip", ipAddress},
                      {"domain", domain.toString()},
                      {"nameserver", nameserver},
                      {"account", account},
@@ -699,7 +740,7 @@ bool RemoteBackend::createSecondaryDomain(const string& ip, const DNSName& domai
   return this->send(query) && this->recv(answer);
 }
 
-bool RemoteBackend::replaceRRSet(uint32_t domain_id, const DNSName& qname, const QType& qtype, const vector<DNSResourceRecord>& rrset)
+bool RemoteBackend::replaceRRSet(domainid_t domain_id, const DNSName& qname, const QType& qtype, const vector<DNSResourceRecord>& rrset)
 {
   Json::array json_rrset;
   for (const auto& rr : rrset) {
@@ -714,7 +755,7 @@ bool RemoteBackend::replaceRRSet(uint32_t domain_id, const DNSName& qname, const
 
   Json query = Json::object{
     {"method", "replaceRRSet"},
-    {"parameters", Json::object{{"domain_id", static_cast<double>(domain_id)}, {"qname", qname.toString()}, {"qtype", qtype.toString()}, {"trxid", static_cast<double>(d_trxid)}, {"rrset", json_rrset}}}};
+    {"parameters", Json::object{{"domain_id", domain_id}, {"qname", qname.toString()}, {"qtype", qtype.toString()}, {"trxid", static_cast<double>(d_trxid)}, {"rrset", json_rrset}}}};
 
   Json answer;
   return this->send(query) && this->recv(answer);
@@ -733,7 +774,7 @@ bool RemoteBackend::feedRecord(const DNSResourceRecord& rr, const DNSName& order
   return this->send(query) && this->recv(answer); // XXX FIXME this API should not return 'true' I think -ahu
 }
 
-bool RemoteBackend::feedEnts(int domain_id, map<DNSName, bool>& nonterm)
+bool RemoteBackend::feedEnts(domainid_t domain_id, map<DNSName, bool>& nonterm)
 {
   Json::array nts;
 
@@ -752,7 +793,7 @@ bool RemoteBackend::feedEnts(int domain_id, map<DNSName, bool>& nonterm)
   return this->send(query) && this->recv(answer);
 }
 
-bool RemoteBackend::feedEnts3(int domain_id, const DNSName& domain, map<DNSName, bool>& nonterm, const NSEC3PARAMRecordContent& ns3prc, bool narrow)
+bool RemoteBackend::feedEnts3(domainid_t domain_id, const DNSName& domain, map<DNSName, bool>& nonterm, const NSEC3PARAMRecordContent& ns3prc, bool narrow)
 {
   Json::array nts;
 
@@ -771,7 +812,7 @@ bool RemoteBackend::feedEnts3(int domain_id, const DNSName& domain, map<DNSName,
   return this->send(query) && this->recv(answer);
 }
 
-bool RemoteBackend::startTransaction(const DNSName& domain, int domain_id)
+bool RemoteBackend::startTransaction(const ZoneName& domain, domainid_t domain_id)
 {
   this->d_trxid = time((time_t*)nullptr);
 
@@ -858,7 +899,7 @@ bool RemoteBackend::searchRecords(const string& pattern, size_t maxResults, vect
     rr.qclass = QClass::IN;
     rr.content = stringFromJson(row, "content");
     rr.ttl = row["ttl"].int_value();
-    rr.domain_id = intFromJson(row, "domain_id", -1);
+    rr.domain_id = static_cast<domainid_t>(intFromJson(row, "domain_id", UnknownDomainID));
     if (d_dnssec) {
       rr.auth = (intFromJson(row, "auth", 1) != 0);
     }
@@ -946,11 +987,11 @@ void RemoteBackend::getUnfreshSecondaryInfos(vector<DomainInfo>* domains)
   }
 }
 
-void RemoteBackend::setStale(uint32_t domain_id)
+void RemoteBackend::setStale(domainid_t domain_id)
 {
   Json query = Json::object{
     {"method", "setStale"},
-    {"parameters", Json::object{{"id", static_cast<double>(domain_id)}}}};
+    {"parameters", Json::object{{"id", domain_id}}}};
 
   Json answer;
   if (!this->send(query) || !this->recv(answer)) {
@@ -958,11 +999,11 @@ void RemoteBackend::setStale(uint32_t domain_id)
   }
 }
 
-void RemoteBackend::setFresh(uint32_t domain_id)
+void RemoteBackend::setFresh(domainid_t domain_id)
 {
   Json query = Json::object{
     {"method", "setFresh"},
-    {"parameters", Json::object{{"id", static_cast<double>(domain_id)}}}};
+    {"parameters", Json::object{{"id", domain_id}}}};
 
   Json answer;
   if (!this->send(query) || !this->recv(answer)) {
