@@ -78,7 +78,7 @@ ArgvMap &arg()
 }
 /* END Needed because of deeper dependencies */
 
-// Allows reading/writing ComboAddresses and DNSNames in YAML-cpp
+// Allows reading/writing ComboAddresses and ZoneNames in YAML-cpp
 namespace YAML {
 template<>
 struct convert<ComboAddress> {
@@ -101,16 +101,16 @@ struct convert<ComboAddress> {
 };
 
 template<>
-struct convert<DNSName> {
-  static Node encode(const DNSName& rhs) {
+struct convert<ZoneName> {
+  static Node encode(const ZoneName& rhs) {
     return Node(rhs.toStringRootDot());
   }
-  static bool decode(const Node& node, DNSName& rhs) {
+  static bool decode(const Node& node, ZoneName& rhs) {
     if (!node.IsScalar()) {
       return false;
     }
     try {
-      rhs = DNSName(node.as<string>());
+      rhs = ZoneName(node.as<string>());
       return true;
     } catch(const runtime_error &e) {
       return false;
@@ -165,14 +165,14 @@ struct ixfrdistdomain_t {
 };
 
 // This contains the configuration for each domain
-static map<DNSName, ixfrdistdomain_t> g_domainConfigs;
+static map<ZoneName, ixfrdistdomain_t> g_domainConfigs;
 
 // Map domains and their data
-static LockGuarded<std::map<DNSName, std::shared_ptr<ixfrinfo_t>>> g_soas;
+static LockGuarded<std::map<ZoneName, std::shared_ptr<ixfrinfo_t>>> g_soas;
 
 // Queue of received NOTIFYs, already verified against their primary IPs
 // Lazily implemented as a set
-static LockGuarded<std::set<DNSName>> g_notifiesReceived;
+static LockGuarded<std::set<ZoneName>> g_notifiesReceived;
 
 // Queue of outgoing NOTIFY
 static LockGuarded<NotificationQueue> g_notificationQueue;
@@ -217,7 +217,7 @@ static bool sortSOA(uint32_t i, uint32_t j) {
   return rfc1982LessThan(i, j);
 }
 
-static void cleanUpDomain(const DNSName& domain, const uint16_t& keep, const string& workdir) {
+static void cleanUpDomain(const ZoneName& domain, const uint16_t& keep, const string& workdir) {
   string dir = workdir + "/" + domain.toString();
   vector<uint32_t> zoneVersions;
   auto directoryError = pdns::visit_directory(dir, [&zoneVersions]([[maybe_unused]] ino_t inodeNumber, const std::string_view& name) {
@@ -288,12 +288,12 @@ static void makeIXFRDiff(const records_t& from, const records_t& to, std::shared
 }
 
 /* you can _never_ alter the content of the resulting shared pointer */
-static std::shared_ptr<ixfrinfo_t> getCurrentZoneInfo(const DNSName& domain)
+static std::shared_ptr<ixfrinfo_t> getCurrentZoneInfo(const ZoneName& domain)
 {
   return (*g_soas.lock())[domain];
 }
 
-static void updateCurrentZoneInfo(const DNSName& domain, std::shared_ptr<ixfrinfo_t>& newInfo)
+static void updateCurrentZoneInfo(const ZoneName& domain, std::shared_ptr<ixfrinfo_t>& newInfo)
 {
   auto soas = g_soas.lock();
   (*soas)[domain] = newInfo;
@@ -301,11 +301,11 @@ static void updateCurrentZoneInfo(const DNSName& domain, std::shared_ptr<ixfrinf
   // FIXME: also report zone size?
 }
 
-static void sendNotification(int sock, const DNSName& domain, const ComboAddress& remote, uint16_t notificationId)
+static void sendNotification(int sock, const ZoneName& domain, const ComboAddress& remote, uint16_t notificationId)
 {
   std::vector<std::string> meta;
   std::vector<uint8_t> packet;
-  DNSPacketWriter packetWriter(packet, domain, QType::SOA, 1, Opcode::Notify);
+  DNSPacketWriter packetWriter(packet, domain.operator const DNSName&(), QType::SOA, 1, Opcode::Notify);
   packetWriter.getHeader()->id = notificationId;
   packetWriter.getHeader()->aa = true;
 
@@ -345,7 +345,7 @@ static void communicatorReceiveNotificationAnswers(const int sock4, const int so
       g_log << Logger::Warning << "Received unsuccessful notification report for '" << packet.qdomain << "' from " << from.toStringWithPort() << ", error: " << RCode::to_s(packet.d.rcode) << endl;
     }
 
-    if (g_notificationQueue.lock()->removeIf(from, packet.d.id, packet.qdomain)) {
+    if (g_notificationQueue.lock()->removeIf(from, packet.d.id, ZoneName(packet.qdomain))) {
       g_log << Logger::Notice << "Removed from notification list: '" << packet.qdomain << "' to " << from.toStringWithPort() << " " << (packet.d.rcode != 0 ? RCode::to_s(packet.d.rcode) : "(was acknowledged)") << endl;
     }
     else {
@@ -356,7 +356,7 @@ static void communicatorReceiveNotificationAnswers(const int sock4, const int so
 
 static void communicatorSendNotifications(const int sock4, const int sock6)
 {
-  DNSName domain;
+  ZoneName domain;
   string destinationIp;
   uint16_t notificationId = 0;
   bool purged{false};
@@ -397,28 +397,28 @@ static void communicatorThread()
     communicatorSendNotifications(sock4, sock6);
     std::this_thread::sleep_for(std::chrono::seconds(1));
   }
-  if (sock4 > 0) {
+  if (sock4 >= 0) {
     closesocket(sock4);
   }
-  if (sock6 > 0) {
+  if (sock6 >= 0) {
     closesocket(sock6);
   }
 }
 
 static void updateThread(const string& workdir, const uint16_t& keep, const uint16_t& axfrTimeout, const uint16_t& soaRetry, const uint32_t axfrMaxRecords) { // NOLINT(readability-function-cognitive-complexity) 13400 https://github.com/PowerDNS/pdns/issues/13400 Habbie:  ixfrdist: reduce complexity
   setThreadName("ixfrdist/update");
-  std::map<DNSName, time_t> lastCheck;
+  std::map<ZoneName, time_t> lastCheck;
 
   // Initialize the serials we have
   for (const auto &domainConfig : g_domainConfigs) {
-    DNSName domain = domainConfig.first;
+    ZoneName domain = domainConfig.first;
     lastCheck[domain] = 0;
     string dir = workdir + "/" + domain.toString();
     try {
       g_log<<Logger::Info<<"Trying to initially load domain "<<domain<<" from disk"<<endl;
       auto serial = getSerialFromDir(dir);
       shared_ptr<const SOARecordContent> soa;
-      uint32_t soaTTL;
+      uint32_t soaTTL{0};
       {
         string fname = workdir + "/" + domain.toString() + "/" + std::to_string(serial);
         loadSOAFromDisk(domain, fname, soa, soaTTL);
@@ -464,7 +464,7 @@ static void updateThread(const string& workdir, const uint16_t& keep, const uint
         break;
       }
 
-      DNSName domain = domainConfig.first;
+      ZoneName domain = domainConfig.first;
       shared_ptr<const SOARecordContent> current_soa;
       const auto& zoneInfo = getCurrentZoneInfo(domain);
       if (zoneInfo != nullptr) {
@@ -625,7 +625,8 @@ static ResponseType maybeHandleNotify(const MOADNSParser& mdp, const ComboAddres
 
   g_log<<Logger::Info<<logPrefix<<"NOTIFY for "<<mdp.d_qname<<"|"<<QType(mdp.d_qtype).toString()<<" "<< Opcode::to_s(mdp.d_header.opcode) <<" from "<<saddr.toStringWithPort()<<endl;
 
-  auto found = g_domainConfigs.find(mdp.d_qname);
+  ZoneName zonename(mdp.d_qname);
+  auto found = g_domainConfigs.find(zonename);
   if (found == g_domainConfigs.end()) {
     g_log<<Logger::Info<<("Domain name '" + mdp.d_qname.toLogString() + "' is not configured for notification")<<endl;
     return ResponseType::RefusedQuery;
@@ -643,12 +644,12 @@ static ResponseType maybeHandleNotify(const MOADNSParser& mdp, const ComboAddres
   }
 
   if (primaryFound) {
-    g_notifiesReceived.lock()->insert(mdp.d_qname);
+    g_notifiesReceived.lock()->insert(zonename);
 
     if (!found->second.notify.empty()) {
       for (const auto& address : found->second.notify) {
         g_log << Logger::Debug << logPrefix << "Queuing notification for " << mdp.d_qname << " to " << address.toStringWithPort() << std::endl;
-        g_notificationQueue.lock()->add(mdp.d_qname, address);
+        g_notificationQueue.lock()->add(zonename, address);
       }
     }
     return ResponseType::EmptyNoError;
@@ -680,12 +681,13 @@ static ResponseType checkQuery(const MOADNSParser& mdp, const ComboAddress& sadd
     }
 
     {
-      if (g_domainConfigs.find(mdp.d_qname) == g_domainConfigs.end()) {
+      ZoneName zonename(mdp.d_qname);
+      if (g_domainConfigs.find(zonename) == g_domainConfigs.end()) {
         info_msg.push_back("Domain name '" + mdp.d_qname.toLogString() + "' is not configured for distribution");
         ret = ResponseType::RefusedQuery;
       }
       else {
-        const auto zoneInfo = getCurrentZoneInfo(mdp.d_qname);
+        const auto zoneInfo = getCurrentZoneInfo(zonename);
         if (zoneInfo == nullptr) {
           info_msg.emplace_back("Domain has not been transferred yet");
           ret = ResponseType::RefusedQuery;
@@ -735,7 +737,7 @@ static bool makeEmptyNoErrorPacket(const MOADNSParser& mdp, vector<uint8_t>& pac
  */
 static bool makeSOAPacket(const MOADNSParser& mdp, vector<uint8_t>& packet) {
 
-  auto zoneInfo = getCurrentZoneInfo(mdp.d_qname);
+  auto zoneInfo = getCurrentZoneInfo(ZoneName(mdp.d_qname));
   if (zoneInfo == nullptr) {
     return false;
   }
@@ -872,9 +874,10 @@ static bool handleAXFR(int fd, const MOADNSParser& mdp) {
      until we release it.
   */
 
-  g_stats.incrementAXFRinQueries(mdp.d_qname);
+  ZoneName zonename(mdp.d_qname);
+  g_stats.incrementAXFRinQueries(zonename);
 
-  auto zoneInfo = getCurrentZoneInfo(mdp.d_qname);
+  auto zoneInfo = getCurrentZoneInfo(zonename);
   if (zoneInfo == nullptr) {
     return false;
   }
@@ -912,9 +915,10 @@ static bool handleIXFR(int fd, const MOADNSParser& mdp, const shared_ptr<const S
      until we release it.
   */
 
-  g_stats.incrementIXFRinQueries(mdp.d_qname);
+  ZoneName zonename(mdp.d_qname);
+  g_stats.incrementIXFRinQueries(zonename);
 
-  auto zoneInfo = getCurrentZoneInfo(mdp.d_qname);
+  auto zoneInfo = getCurrentZoneInfo(zonename);
   if (zoneInfo == nullptr) {
     return false;
   }
@@ -1075,12 +1079,12 @@ try
      * Let's not complicate this with IXFR over UDP (and looking if we need to truncate etc).
      * Just send the current SOA and let the client try over TCP
      */
-    g_stats.incrementSOAinQueries(mdp.d_qname); // FIXME: this also counts IXFR queries (but the response is the same as to a SOA query)
+    g_stats.incrementSOAinQueries(ZoneName(mdp.d_qname)); // FIXME: this also counts IXFR queries (but the response is the same as to a SOA query)
     makeSOAPacket(mdp, packet);
   } else if (respt == ResponseType::EmptyNoError) {
     makeEmptyNoErrorPacket(mdp, packet);
   } else if (respt == ResponseType::RefusedQuery) {
-    g_stats.incrementUnknownDomainInQueries(mdp.d_qname);
+    g_stats.incrementUnknownDomainInQueries(ZoneName(mdp.d_qname));
     makeRefusedPacket(mdp, packet);
   } else if (respt == ResponseType::RefusedOpcode) {
     g_stats.incrementNotImplemented(mdp.d_header.opcode);
@@ -1125,7 +1129,7 @@ static void handleTCPRequest(int fd, boost::any&) {
   }
 
   {
-    std::lock_guard<std::mutex> lg(g_tcpRequestFDsMutex);
+    auto lock = std::scoped_lock(g_tcpRequestFDsMutex);
     g_tcpRequestFDs.push({cfd, saddr});
   }
   g_tcpHandlerCV.notify_one();
@@ -1224,8 +1228,8 @@ static void tcpWorker(int tid) {
         for (auto &answer : mdp.d_answers) {
           // from dnsparser.hh:
           // typedef vector<pair<DNSRecord, uint16_t > > answers_t;
-          if (answer.first.d_type == QType::SOA && answer.first.d_place == DNSResourceRecord::AUTHORITY) {
-            clientSOA = getRR<SOARecordContent>(answer.first);
+          if (answer.d_type == QType::SOA && answer.d_place == DNSResourceRecord::AUTHORITY) {
+            clientSOA = getRR<SOARecordContent>(answer);
             if (clientSOA != nullptr) {
               break;
             }
@@ -1391,7 +1395,7 @@ static bool parseAndCheckConfig(const string& configpath, YAML::Node& config) {
           retval = false;
           continue;
         }
-        domain["domain"].as<DNSName>();
+        domain["domain"].as<ZoneName>();
       } catch (const runtime_error &e) {
         g_log<<Logger::Error<<"Unable to read domain '"<<domain["domain"].as<string>()<<"': "<<e.what()<<endl;
       }
@@ -1556,12 +1560,12 @@ static std::optional<IXFRDistConfiguration> parseConfiguration(int argc, char**
     for (auto const &domain : config["domains"]) {
       set<ComboAddress> s;
       s.insert(domain["master"].as<ComboAddress>());
-      g_domainConfigs[domain["domain"].as<DNSName>()].primaries = s;
+      g_domainConfigs[domain["domain"].as<ZoneName>()].primaries = s;
       if (domain["max-soa-refresh"].IsDefined()) {
-        g_domainConfigs[domain["domain"].as<DNSName>()].maxSOARefresh = domain["max-soa-refresh"].as<uint32_t>();
+        g_domainConfigs[domain["domain"].as<ZoneName>()].maxSOARefresh = domain["max-soa-refresh"].as<uint32_t>();
       }
       if (domain["notify"].IsDefined()) {
-        auto& listset = g_domainConfigs[domain["domain"].as<DNSName>()].notify;
+        auto& listset = g_domainConfigs[domain["domain"].as<ZoneName>()].notify;
         if (domain["notify"].IsScalar()) {
           auto remote = domain["notify"].as<std::string>();
           try {
@@ -1582,7 +1586,7 @@ static std::optional<IXFRDistConfiguration> parseConfiguration(int argc, char**
           }
         }
       }
-      g_stats.registerDomain(domain["domain"].as<DNSName>());
+      g_stats.registerDomain(domain["domain"].as<ZoneName>());
     }
 
     for (const auto &addr : config["acl"].as<vector<string>>()) {
@@ -1621,6 +1625,11 @@ static std::optional<IXFRDistConfiguration> parseConfiguration(int argc, char**
           int s = SSocket(addr.sin4.sin_family, stype, 0);
           setNonBlocking(s);
           setReuseAddr(s);
+          if (addr.isIPv6()) {
+            int one = 1;
+            (void)setsockopt(s, IPPROTO_IPV6, IPV6_V6ONLY, &one, sizeof(one));
+          }
+
           SBind(s, addr);
           if (stype == SOCK_STREAM) {
             SListen(s, 30); // TODO make this configurable
