@@ -1,4 +1,7 @@
+#ifndef BOOST_TEST_DYN_LINK
 #define BOOST_TEST_DYN_LINK
+#endif
+
 #define BOOST_TEST_NO_MAIN
 
 #ifdef HAVE_CONFIG_H
@@ -21,7 +24,7 @@ BOOST_AUTO_TEST_CASE(test_xfrIP6) {
         loopback6.append(15, 0);
         loopback6.append(1,1);
         BOOST_CHECK_EQUAL(makeHexDump(rawIPv6), makeHexDump(loopback6));
-        
+
         RecordTextReader rtr2("2a01:4f8:d12:1880::5");
         rtr2.xfrIP6(rawIPv6);
         string ip6("\x2a\x01\x04\xf8\x0d\x12\x18\x80\x00\x00\x00\x00\x00\x00\x00\x05", 16);
@@ -89,6 +92,14 @@ BOOST_AUTO_TEST_CASE(test_xfrSvcParamKeyVals_alpn) {
         source="key1=\\002h2\\002h3foobar"; // extra data
         RecordTextReader rtr5(source);
         BOOST_CHECK_THROW(rtr5.xfrSvcParamKeyVals(v), RecordTextException);
+
+        source="key1"; // no data
+        RecordTextReader rtr6(source);
+        BOOST_CHECK_THROW(rtr6.xfrSvcParamKeyVals(v), RecordTextException);
+
+        source="key1=\\000"; // no data
+        RecordTextReader rtr7(source);
+        BOOST_CHECK_THROW(rtr7.xfrSvcParamKeyVals(v), RecordTextException);
 }
 
 BOOST_AUTO_TEST_CASE(test_xfrSvcParamKeyVals_mandatory) {
@@ -177,12 +188,37 @@ BOOST_AUTO_TEST_CASE(test_xfrSvcParamKeyVals_no_default_alpn) {
 
         // Generic
         v.clear();
-        source = "key2";
-        RecordTextReader rtr3(source);
+        RecordTextReader rtr3("key2");
         rtr3.xfrSvcParamKeyVals(v);
         BOOST_CHECK_EQUAL(v.size(), 1U);
         k = v.begin()->getKey();
         BOOST_CHECK(k == SvcParam::no_default_alpn);
+
+        v.clear();
+        RecordTextReader rtr4("key2=");
+        BOOST_CHECK_THROW(rtr4.xfrSvcParamKeyVals(v), RecordTextException);
+
+        v.clear();
+        RecordTextReader rtr5("key2 ipv4hint=1.2.3.4");
+        rtr5.xfrSvcParamKeyVals(v);
+        BOOST_CHECK_EQUAL(v.size(), 2U);
+        k = v.begin()->getKey();
+        BOOST_CHECK(k == SvcParam::no_default_alpn);
+
+        v.clear();
+        RecordTextReader rtr6("ipv4hint=1.2.3.4 key2");
+        rtr6.xfrSvcParamKeyVals(v);
+        BOOST_CHECK_EQUAL(v.size(), 2U);
+        k = v.begin()->getKey();
+        BOOST_CHECK(k == SvcParam::no_default_alpn);
+
+        v.clear();
+        RecordTextReader rtr7("key2=port=123 ipv4hint=1.2.3.4");
+        BOOST_CHECK_THROW(rtr7.xfrSvcParamKeyVals(v), RecordTextException);
+
+        v.clear();
+        RecordTextReader rtr8("key2=port=123");
+        BOOST_CHECK_THROW(rtr8.xfrSvcParamKeyVals(v), RecordTextException);
 }
 
 BOOST_AUTO_TEST_CASE(test_xfrSvcParamKeyVals_ipv4hint) {
@@ -260,6 +296,23 @@ BOOST_AUTO_TEST_CASE(test_xfrSvcParamKeyVals_ipv4hint) {
         source = "key4=\\192\\000\\222"; // Wrong number of octets
         RecordTextReader rtr7(source);
         BOOST_CHECK_THROW(rtr7.xfrSvcParamKeyVals(v), RecordTextException);
+
+        v.clear();
+        RecordTextReader rtr8("key4=");  // must have a value
+        BOOST_CHECK_THROW(rtr8.xfrSvcParamKeyVals(v), RecordTextException);
+
+        v.clear();
+        RecordTextReader rtr9("ipv4hint=");  // must have a value
+        BOOST_CHECK_THROW(rtr9.xfrSvcParamKeyVals(v), RecordTextException);
+
+        v.clear();
+        RecordTextReader rtr10("ipv4hint=auto");  // special value
+        rtr10.xfrSvcParamKeyVals(v);
+        BOOST_CHECK_EQUAL(v.size(), 1U);
+        k = v.begin()->getKey();
+        BOOST_CHECK(k == SvcParam::ipv4hint);
+        BOOST_CHECK_EQUAL(v.begin()->getIPHints().size(), 0U);
+        BOOST_CHECK_EQUAL(v.begin()->getAutoHint(), true);
 }
 
 BOOST_AUTO_TEST_CASE(test_xfrSvcParamKeyVals_ipv6hint) {
@@ -336,6 +389,14 @@ BOOST_AUTO_TEST_CASE(test_xfrSvcParamKeyVals_ipv6hint) {
         source = "key6=\\040\\001\\015\\270\\000\\123\\000\\000\\000\\000\\000\\000\\000\\000\\000"; // wrong number of octets
         RecordTextReader rtr7(source);
         BOOST_CHECK_THROW(rtr7.xfrSvcParamKeyVals(v), RecordTextException);
+
+        v.clear();
+        RecordTextReader rtr8("key6="); // must have a value
+        BOOST_CHECK_THROW(rtr8.xfrSvcParamKeyVals(v), RecordTextException);
+
+        v.clear();
+        RecordTextReader rtr9("ipv6hint=");  // must have a value
+        BOOST_CHECK_THROW(rtr9.xfrSvcParamKeyVals(v), RecordTextException);
 }
 
 BOOST_AUTO_TEST_CASE(test_xfrSvcParamKeyVals_port) {
@@ -409,7 +470,10 @@ BOOST_AUTO_TEST_CASE(test_xfrSvcParamKeyVals_generic) {
 
         v.clear();
         RecordTextReader rtr3("key666");
-        BOOST_CHECK_THROW(rtr3.xfrSvcParamKeyVals(v), RecordTextException);
+        BOOST_CHECK_NO_THROW(rtr3.xfrSvcParamKeyVals(v));
+        BOOST_CHECK_EQUAL(v.size(), 1U);
+        k = v.begin()->getKey();
+        BOOST_CHECK(k == SvcParam::keyFromString("key666"));
 
         v.clear();
         source = "key666=\"blablabla\"";
@@ -487,6 +551,20 @@ BOOST_AUTO_TEST_CASE(test_xfrSvcParamKeyVals_multiple) {
         RecordTextWriter rtw(target);
         rtw.xfrSvcParamKeyVals(v);
         BOOST_CHECK_EQUAL(target, "mandatory=alpn alpn=h2,h3 ipv4hint=192.0.2.1,192.0.2.2 ech=\"dG90YWxseSBib2d1cyBlY2hjb25maWcgdmFsdWU=\" ipv6hint=2001:db8::1 key666=\"foobar\"");
+
+        v.clear();
+        RecordTextReader rtr2("mandatory=alpn key666"); // generic key without value at the end of the string
+        BOOST_CHECK_NO_THROW(rtr2.xfrSvcParamKeyVals(v));
+        BOOST_CHECK_EQUAL(v.size(), 2U);
+
+        v.clear();
+        RecordTextReader rtr3("key666 key677=\"foo\" mandatory=alpn"); // generic key without value -not- at the end of the string
+        BOOST_CHECK_NO_THROW(rtr3.xfrSvcParamKeyVals(v));
+        BOOST_CHECK_EQUAL(v.size(), 3U);
+
+        v.clear();
+        RecordTextReader rtr4("mandatory= key666"); // non-generic key without value -not- at the end of the string
+        BOOST_CHECK_THROW(rtr4.xfrSvcParamKeyVals(v), RecordTextException);
 }
 
 BOOST_AUTO_TEST_CASE(test_xfrSvcParamKeyVals_ech) {
@@ -516,6 +594,14 @@ BOOST_AUTO_TEST_CASE(test_xfrSvcParamKeyVals_ech) {
         BOOST_CHECK(k == SvcParam::ech);
         val = v.begin()->getECH();
         BOOST_CHECK_EQUAL(val, "echconfig");
+
+        v.clear();
+        RecordTextReader rtr3("key5");
+        BOOST_CHECK_THROW(rtr3.xfrSvcParamKeyVals(v), RecordTextException);
+
+        v.clear();
+        RecordTextReader rtr4("ech=\"\"");
+        BOOST_CHECK_THROW(rtr4.xfrSvcParamKeyVals(v), RecordTextException);
 }
 
 BOOST_AUTO_TEST_CASE(test_xfrNodeOrLocatorID) {
